"""Integral domains."""

from enum import Enum


class Domain(Enum):
    """A domain of an integral."""

    EdgeAdjacentTriangleAndQuadrilateral = 0
    Interval = 1
    Quadrilateral = 2
    TriangularPrism = 3
    VertexAdjacentTriangles = 4
    VertexAdjacentTriangleAndQuadrilateral = 5
    SquareBasedPyramid = 6
    Triangle = 7
    Hexahedron = 8
    Tetrahedron = 9
    EdgeAdjacentTriangles = 10
    EdgeAdjacentQuadrilaterals = 11
    VertexAdjacentQuadrilaterals = 12



"""vertex quadrature quadrature rule."""

import numpy as np
import numpy.typing as npt
from quadraturerules.domain import Domain
import typing


def vertex_quadrature(
    domain: Domain,
    order: int,
) -> typing.Tuple[npt.NDArray[np.float64], npt.NDArray[np.float64]]:
    """Get a vertex quadrature quadrature rule."""
    match domain:
        case Domain.EdgeAdjacentTriangleAndQuadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Interval:
            match order:
                case 1:
                    return np.array([[1.0, 0.0], [0.0, 1.0]]), np.array([0.5, 0.5])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Quadrilateral:
            match order:
                case 1:
                    return np.array(
                        [
                            [1.0, 0.0, 0.0, 0.0],
                            [0.0, 1.0, 0.0, 0.0],
                            [0.0, 0.0, 1.0, 0.0],
                            [0.0, 0.0, 0.0, 1.0],
                        ]
                    ), np.array([0.25, 0.25, 0.25, 0.25])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.TriangularPrism:
            match order:
                case 1:
                    return np.array(
                        [
                            [1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
                        ]
                    ), np.array(
                        [
                            0.16666666666666666,
                            0.16666666666666666,
                            0.16666666666666666,
                            0.16666666666666666,
                            0.16666666666666666,
                            0.16666666666666666,
                        ]
                    )
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentTriangles:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentTriangleAndQuadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.SquareBasedPyramid:
            match order:
                case 1:
                    return np.array(
                        [
                            [1.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 1.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 1.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 1.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 1.0],
                        ]
                    ), np.array([0.2, 0.2, 0.2, 0.2, 0.2])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Triangle:
            match order:
                case 1:
                    return np.array([[1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]]), np.array(
                        [0.3333333333333333, 0.3333333333333333, 0.3333333333333333]
                    )
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Hexahedron:
            match order:
                case 1:
                    return np.array(
                        [
                            [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
                            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
                        ]
                    ), np.array([0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Tetrahedron:
            match order:
                case 1:
                    return np.array(
                        [
                            [1.0, 0.0, 0.0, 0.0],
                            [0.0, 1.0, 0.0, 0.0],
                            [0.0, 0.0, 1.0, 0.0],
                            [0.0, 0.0, 0.0, 1.0],
                        ]
                    ), np.array([0.25, 0.25, 0.25, 0.25])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.EdgeAdjacentTriangles:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.EdgeAdjacentQuadrilaterals:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentQuadrilaterals:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")



"""Quadrature rules."""

from enum import Enum as _Enum
import typing as _typing
import numpy.typing as _npt
import numpy as _np

from quadraturerules import rules
from quadraturerules.domain import Domain


class QuadratureRule(_Enum):
    """A quadrature rule family."""

    GaussLegendre = 1
    GaussLobattoLegendre = 3
    HammerMarloweStroud = 6
    SauterSchwab = 7
    XiaoGimbutas = 2
    CentroidQuadrature = 5
    VertexQuadrature = 4


def single_integral_quadrature(
    rtype: QuadratureRule,
    domain: Domain,
    order: int,
) -> _typing.Tuple[_npt.NDArray[_np.float64], _npt.NDArray[_np.float64]]:
    """Get a quadrature rule for a single integral."""
    match rtype:
        case QuadratureRule.GaussLegendre:
            return rules.gauss_legendre(domain, order)
        case QuadratureRule.GaussLobattoLegendre:
            return rules.gauss_lobatto_legendre(domain, order)
        case QuadratureRule.HammerMarloweStroud:
            return rules.hammer_marlowe_stroud(domain, order)
        case QuadratureRule.XiaoGimbutas:
            return rules.xiao_gimbutas(domain, order)
        case QuadratureRule.CentroidQuadrature:
            return rules.centroid_quadrature(domain, order)
        case QuadratureRule.VertexQuadrature:
            return rules.vertex_quadrature(domain, order)
        case _:
            raise ValueError(f"Unsupported rule for single integral: {rtype}")


def double_integral_quadrature(
    rtype: QuadratureRule,
    domain: Domain,
    order: int,
) -> _typing.Tuple[_npt.NDArray[_np.float64], _npt.NDArray[_np.float64], _npt.NDArray[_np.float64]]:
    """Get a quadrature rule for a double integral."""
    match rtype:
        case QuadratureRule.SauterSchwab:
            return rules.sauter_schwab(domain, order)
        case _:
            raise ValueError(f"Unsupported rule for double integral: {rtype}")



"""centroid quadrature quadrature rule."""

import numpy as np
import numpy.typing as npt
from quadraturerules.domain import Domain
import typing


def centroid_quadrature(
    domain: Domain,
    order: int,
) -> typing.Tuple[npt.NDArray[np.float64], npt.NDArray[np.float64]]:
    """Get a centroid quadrature quadrature rule."""
    match domain:
        case Domain.EdgeAdjacentTriangleAndQuadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Interval:
            match order:
                case 1:
                    return np.array([[0.5, 0.5]]), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Quadrilateral:
            match order:
                case 1:
                    return np.array([[0.25, 0.25, 0.25, 0.25]]), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.TriangularPrism:
            match order:
                case 1:
                    return np.array(
                        [
                            [
                                0.16666666666666666,
                                0.16666666666666666,
                                0.16666666666666666,
                                0.16666666666666666,
                                0.16666666666666666,
                                0.16666666666666666,
                            ]
                        ]
                    ), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentTriangles:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentTriangleAndQuadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.SquareBasedPyramid:
            match order:
                case 1:
                    return np.array([[0.2, 0.2, 0.2, 0.2, 0.2]]), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Triangle:
            match order:
                case 1:
                    return np.array(
                        [[0.3333333333333333, 0.3333333333333333, 0.3333333333333333]]
                    ), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Hexahedron:
            match order:
                case 1:
                    return np.array(
                        [[0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125]]
                    ), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Tetrahedron:
            match order:
                case 1:
                    return np.array([[0.25, 0.25, 0.25, 0.25]]), np.array([1.0])
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.EdgeAdjacentTriangles:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.EdgeAdjacentQuadrilaterals:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.VertexAdjacentQuadrilaterals:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")



import pytest
import numpy as np
from quadraturerules import Domain, QuadratureRule, single_integral_quadrature


@pytest.mark.parametrize("order", range(1, 10))
def test_integral_of_polynomial(order):
    # f = x ** (2 * order - 1)
    # integral(f, x) = x ** (2 * order) / (2 * order)
    # integral(f, x=0..1) = 0.5 / order
    pts, wts = single_integral_quadrature(
        QuadratureRule.GaussLobattoLegendre, Domain.Interval, order
    )
    integral = sum(w * p[1] ** (2 * order - 1) for p, w in zip(pts, wts))
    assert np.isclose(integral, 0.5 / order)



import pytest
import numpy as np
from quadraturerules import Domain, QuadratureRule, single_integral_quadrature


@pytest.mark.parametrize("order", [1, 2, 3])
def test_sum_weights(order):
    assert np.isclose(
        sum(single_integral_quadrature(QuadratureRule.GaussLegendre, Domain.Interval, order)[1]),
        1.0,
    )


@pytest.mark.parametrize("order", range(1, 10))
def test_integral_of_polynomial(order):
    # f = x ** (2 * order - 1)
    # integral(f, x) = x ** (2 * order) / (2 * order)
    # integral(f, x=0..1) = 0.5 / order
    pts, wts = single_integral_quadrature(QuadratureRule.GaussLegendre, Domain.Interval, order)
    integral = sum(w * p[1] ** (2 * order - 1) for p, w in zip(pts, wts))
    assert np.isclose(integral, 0.5 / order)



"""Definitions of quadrature rules."""

from quadraturerules.rules.gauss_legendre import gauss_legendre
from quadraturerules.rules.gauss_lobatto_legendre import gauss_lobatto_legendre
from quadraturerules.rules.hammer_marlowe_stroud import hammer_marlowe_stroud
from quadraturerules.rules.sauter_schwab import sauter_schwab
from quadraturerules.rules.xiao_gimbutas import xiao_gimbutas
from quadraturerules.rules.centroid_quadrature import centroid_quadrature
from quadraturerules.rules.vertex_quadrature import vertex_quadrature
